from django import forms
from os import path
from collections import namedtuple
from django.core.files.storage import default_storage

from markdownx.forms import ImageForm
from markdownx.exceptions import MarkdownxImageUploadError
from markdownx.settings import _mdx

from markdownx.settings import (
    MARKDOWNX_MEDIA_PATH,
    MARKDOWNX_UPLOAD_CONTENT_TYPES,
    MARKDOWNX_UPLOAD_MAX_SIZE,
    # MARKDOWNX_USE_ORIGINAL_IMAGE_NAME,
)
# sas 2020-09-27 : should place in markdownx.settings above but placing here so there are fewer changes
MARKDOWNX_USE_ORIGINAL_IMAGE_NAME = _mdx('USE_ORIGINAL_IMAGE_NAME', False)


# markdownx overrides for forms
class MarkdownImageUploadForm(ImageForm):

    # sas 2020-09-15 : adding field to sanitize the input for the field image prefix if given
    field_image_prefix = forms.CharField(required=False)

    def _save(self, image, file_name, commit):
        """
        Final saving process, called internally after processing tasks are complete.

        :param image: Prepared image
        :type image: django.core.files.uploadedfile.InMemoryUploadedFile
        :param file_name: Name of the file using which the image is to be saved.
        :type file_name: str
        :param commit: If ``True``, the image is saved onto the disk.
        :type commit: bool
        :return: URL of the uploaded image ``commit=True``, otherwise a namedtuple
                 of ``(path, image)`` where ``path`` is the absolute path generated
                 for saving the file, and ``image`` is the prepared image.
        :rtype: str, namedtuple

        NOTE: copied from original form with 2 modifications below
            1) we do not store a unique file name; if the filename exists we skip and just return the url
            2) we will inject the new field_image_prefix path if it is provided
                NOTE: provided by model and passed via ajax javascript to view

        """
        # Defining a universally unique name for the file
        # to be saved on the disk.

        # NOTE: this takes away the ability for content creators to control the file name
        #   or quickly return an existing image link.  Altering to give better control and functionality over file names
        #   add MARKDOWNX_USE_ORIGINAL_IMAGE_NAME = True to change the default functionality
        # sas 2020-09-27 : added functionality to use the original file name and just return link if exists
        if MARKDOWNX_USE_ORIGINAL_IMAGE_NAME:
            unique_file_name = file_name
        else:
            unique_file_name = self.get_unique_file_name(file_name)

        # sas 2020-09-15 : reading the sanitized image prefix or empty string and appending to path
        field_image_prefix = self.cleaned_data['field_image_prefix'] or ''
        full_path = path.join(MARKDOWNX_MEDIA_PATH, field_image_prefix, unique_file_name)

        if commit:
            # sas 2020-09-27 : we only want to save if the image doesn't already exist; otherwise we just return the url
            if MARKDOWNX_USE_ORIGINAL_IMAGE_NAME and default_storage.exists(full_path):
                pass
            else:
                default_storage.save(full_path, image)
            return default_storage.url(full_path)

        # If `commit is False`, return the path and in-memory image.
        image_data = namedtuple('image_data', ['path', 'image'])
        return image_data(path=full_path, image=image)

    def clean(self):
        """
        Checks the upload against allowed extensions and maximum size.

        :return: Upload

        NOTE: copied from original form with 1 modification below
            1) commented setting to just image so we can keep the clean_data for our new field override too
        """
        upload = self.cleaned_data.get('image')

        # -----------------------------------------------
        # See comments in `self._error_templates` for
        # additional information on each error.
        # -----------------------------------------------
        if not upload:
            raise MarkdownxImageUploadError.not_uploaded()

        content_type = upload.content_type
        file_size = upload.size

        if content_type not in MARKDOWNX_UPLOAD_CONTENT_TYPES:

            raise MarkdownxImageUploadError.unsupported_format()

        elif file_size > MARKDOWNX_UPLOAD_MAX_SIZE:

            raise MarkdownxImageUploadError.invalid_size(
                current=file_size,
                expected=MARKDOWNX_UPLOAD_MAX_SIZE
            )
        # sas 2020-09-15 : removing the upload return which strips other vars like the new field image prefix
        # return upload


class LoginForm(forms.Form):

    login = forms.CharField(max_length=255)
    password = forms.CharField(widget=forms.PasswordInput())
    target = forms.CharField()



from . import views
from django.urls import path

urlpatterns = [
    # path('simple/', views.PostList.as_view(), name='blog_home'),
    # path('simple/<slug:slug>/', views.PostDetail.as_view(), name='blog_detail'),
    path('', views.PostList.as_view(), name='blog_home'),
    path('<slug:slug>/', views.PostDetail.as_view(), name='blog_detail'),
]



"""cms URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.11/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
from django.urls import path
from . import views

urlpatterns = [
    path('content/', views.ContentApi.as_view(), name='cms_content'),
    path('login', views.LoginFormView.as_view(), name="cms_login"),
    path('logout', views.LogoutView.as_view(), name="cms_logout"),
    path('auth', views.AuthenticateView.as_view(), name="cms_authenticate"),
    # sas 2020-09-27 : attempting to override the markdownx url and apply our overridden view/form instead
    # NOTE: don't forget to copy the js changes into static for transferring the new field to the view
    path('markdownx/upload/', views.MarkdownImageUploadView.as_view(), name='markdownx_upload')
]



from django.core.management.base import BaseCommand
from django.utils.crypto import get_random_string
from argparse import RawTextHelpFormatter
from django.conf import settings
import pathlib
import re
from datetime import datetime
import shutil


class Command(BaseCommand):
    help = """
        usage: ./manage.py secret_key [option]
        --------------------------------------
        example: ./manage.py secret_key generate
        example: ./manage.py secret_key get
        example: ./manage.py secret_key set
        example: ./manage.py secret_key set "4h0u&vk3l(s2t#@c(-%s5_fa_=1ww02s_f6+g-_^^^s+^y8)bp"

        options
        --------
        generate - generates a new secret key and prints it to the screen
        get - prints the current loaded settings secret_key value
        set - sets the .secret_key file to value passed or generated value and prints it to the screen
    """

    def create_parser(self, *args, **kwargs):
        parser = super(Command, self).create_parser(*args, **kwargs)
        parser.formatter_class = RawTextHelpFormatter
        return parser

    def add_arguments(self, parser):
        parser.add_argument('option', nargs='+', type=str)

    @staticmethod
    def generate_secret():
        chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
        return get_random_string(50, chars)

    @staticmethod
    def get_secret():
        return settings.SECRET_KEY

    def set_secret(self, options):
        # if we were passed a key use it otherwise generate one
        if len(options) >= 2:
            secret_key = str(options[1])
        else:
            secret_key = self.generate_secret()
        local_path = pathlib.Path()
        # print(f'local path: {local_path}')
        settings_path = local_path / "docroot" / "settings.py"
        all_settings = settings_path.read_text()
        # find where we are replacing secret_key and replace the value
        qt_pattern = re.compile(r"\s*SECRET_KEY\s*=\s*'(\S*)'")
        new_string = self.replace_pattern_group(all_settings, qt_pattern, secret_key)
        dbqt_pattern = re.compile(r'\s*SECRET_KEY\s*=\s*"(\S*)"')
        new_string = self.replace_pattern_group(new_string, dbqt_pattern, secret_key)
        # print(f'new settings: {new_string}')
        # backup the old file before we mess with it
        datetime_ext = datetime.now().strftime("%Y%m%d-%H%M")
        new_file_name = settings_path.name + "." + datetime_ext
        shutil.copy(settings_path, settings_path.parent / new_file_name)
        settings_path.write_text(new_string)
        return secret_key

    def handle(self, *args, **options):
        if "generate" in options['option']:
            self.stdout.write(self.style.SUCCESS('%s' % self.generate_secret()))
        elif "get" in options['option']:
            self.stdout.write(self.style.SUCCESS('%s' % self.get_secret()))
        elif "set" in options['option']:
            self.stdout.write(self.style.WARNING('WARNING: You must restart the server to apply the new secret_key'))
            self.stdout.write(self.style.SUCCESS(self.set_secret(options['option'])))
        else:
            self.stdout.write(self.style.SUCCESS(self.help))

    @staticmethod
    def replace_pattern_group(string, pattern, value):
        last_match = 0
        new_string = ""
        for match in re.finditer(pattern, string):
            # print(match.group(1))
            group_start = match.span(1)[0]
            group_end = match.span(1)[1]
            new_string += string[last_match:group_start]
            new_string += value
            last_match = group_end
        new_string += string[last_match:]
        return new_string



from django.contrib import admin
from markdownx.admin import MarkdownxModelAdmin
from .models import MarkdownPost


@admin.register(MarkdownPost)
class PostAdmin(MarkdownxModelAdmin):
    list_display = ('title', 'slug', 'created_date')
    list_filter = ('modified_by',)
    search_fields = ['title', 'content']
    prepopulated_fields = {'slug': ('title',)}
    readonly_fields = ('modified_by',)

    def save_model(self, request, obj, form, change):
        instance = form.save(commit=False)
        instance.modified_by = str(request.user)[:255]
        instance.save()
        form.save_m2m()
        return instance



from django.views import generic
# from django.utils import timezone
from .models import MarkdownPost
import logging

log = logging.getLogger("blog.views")


class PostList(generic.ListView):
    model = MarkdownPost
    # today = timezone.now()
    queryset = MarkdownPost.objects.live().order_by('-modified_date')
    template_name = 'docrootcms_blog/index.html'


class PostDetail(generic.DetailView):
    model = MarkdownPost
    template_name = 'docrootcms_blog/post_detail.html'



# Based off of flat pages but no database needed.  Concept is that we have dynamic
# urls based off of a docroot that extend from a template.  Each developer page is a template
# relative to DOCROOT_URL.  The purpose is to bypass having to enter urls for each page we build
# which would get out of hand quickly.  Furthermore, we would like to dynamically inject the
# data context for the page template to avoid having thousands of view functions that the url calls
# based on a naming convention (<dev_page_name>.data.py).  Templates must be named with the .dt extension
# so we and production web servers know what files to send to us to dynamically build.
import logging

from . import views as cms_views
# from django.http import Http404
# from django.conf import settings
# from django.core.exceptions import MiddlewareNotUsed

log = logging.getLogger("docroot.middleware")


# testing the new way of calling middleware
# make get_response=None to test for < 1.9 (see above)
class DocrootFallbackMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        log.info("DocrootFallbackMiddleware initialized...")
        # todo: use this example and import to disable our middleware based on a settings entry
        # raise MiddlewareNotUsed('DISABLE_MIDDLEWARE is set')

    def __call__(self, request):
        # code to be executed before the view/next middleware is called
        response = self.get_response(request)
        # code to be executed after the view/next middleware is called
        log.debug("DocrootFallbackMiddleware called: " + request.path_info)
        if response.status_code == 404:
            log.debug("got a response of 404 would have done something for: " + request.path_info)
            # will be called by apache for all 404's;
            # first attempt to load a static file (should we skip this if nginx arleady processed? DEBUG=FALSE?

            # attempt to load as static file
            result = cms_views.static(request)
            if result:
                log.debug("result is a static file...")
                response = result

            # attempt to load as template
            if not result:
                result = cms_views.page(request)
                if result:
                    log.debug("result is not none so returning it...")
                    response = result

            # attempt to load an api (determined by extension [.json, .xml etc])
            if not result:
                result = cms_views.api(request)
                if result:
                    log.debug("result is not none so returning it...")
                    response = result

        return response



from django.core.management.base import BaseCommand
from argparse import RawTextHelpFormatter
import os
import shutil
import pathlib
from datetime import datetime
from distutils.sysconfig import get_python_lib


class Command(BaseCommand):
    help = """
    usage: ./manage.py docrootcms [option]
    --------------------------------------
    example: ./manage.py docrootcms install
    example: ./manage.py docrootcms test update
    example: ./manage.py docrootcms update
    example: ./manage.py docrootcms develop

    options
    --------
    install - attempts to install the starter files and directories to a new docroot application
    update - attempts to update user files with any configuration changes in settings and urls
    test - creates new version of files that would be changed instead of changing them
    develop - copies the docroot-cms module in the virtual environment to local project for development
    """
    testing = False

    def create_parser(self, *args, **kwargs):
        parser = super(Command, self).create_parser(*args, **kwargs)
        parser.formatter_class = RawTextHelpFormatter
        return parser

    def add_arguments(self, parser):
        parser.add_argument('option', nargs='+', type=str)

    def update(self):
        # look for the lines indicating this is our block and replace them with the current contents
        # otherwise throw an exception and print it as an error
        success_instructions = """
        Successfully updated cms.

        """
        # check that docroot app exists
        # NOTE: we will assume if there is a docroot dir then we already installed
        if not os.path.exists('docroot'):
            return 'docroot directory does not exist; can not update.  Did you mean to install it?'
        else:
            self.stdout.write(f'attempting to update docroot app and test files')
            # get the directory from docrootcms/docroot
            module_path = self.get_module_path()
            # we have a module path so now lets try and copy the docroot_settings_append and docroot_urls_append
            # settings.py changes
            local_path = pathlib.Path()
            print(f'local path: {local_path}')
            settings_path = local_path / "docroot" / "settings.py"
            settings_append_path = module_path / "resources" / "docroot_settings_append.py"
            self.append_or_replace_content(settings_path, settings_append_path)
            # urls.py changes
            urls_path = local_path / "docroot" / "urls.py"
            urls_append_path = module_path / "resources" / "docroot_urls_append.py"
            self.append_or_replace_content(urls_path, urls_append_path)
            # create the resources folder if it doesn't exist and bring all the templates and append files over
            module_resources_path = module_path / 'resources'
            resources_path = local_path / 'docroot' / 'resources'
            if os.path.exists(resources_path):
                shutil.rmtree(resources_path)
                self.stdout.write(self.style.SUCCESS(f"Removed {resources_path}"))
            # copy the module resources to our resources (should contain various files like append index.dt and data.py
            shutil.copytree(module_resources_path, resources_path)
            self.stdout.write(self.style.SUCCESS(f"Copied {module_resources_path} to {resources_path}"))
            module_template_path = module_path / 'templates'
            shutil.copytree(module_template_path, resources_path / 'templates')
            self.stdout.write(self.style.SUCCESS(f"Copied {module_template_path} to {resources_path / 'templates'}"))
            module_test_path = module_path / 'install' / 'files' / 'test'
            try:
                shutil.copytree(module_test_path, resources_path, dirs_exist_ok=True)
                self.stdout.write(self.style.SUCCESS(f"Copied {module_test_path} to {resources_path}"))
            except TypeError:
                # < 3.8 throws type error because the dirs_exist_ok doesn't exist
                # NOTE: trying to delete this every time so hopefully we don't need it anymore; if doesn't work go
                #   back to printing the warning until later
                shutil.copytree(module_test_path, resources_path)
                # print('WARNING: python < 3.8 required version; skipping the replacing of any test files...')

            return success_instructions

    def append_or_replace_content(self, old_path, new_path):
        if os.path.exists(new_path):
            # with open(new_path, 'r') as input_file:
            #     content_block = input_file.read()
            content_block = new_path.read_text()
            content_block = content_block.strip()
            # print(content_block)
            heading_substring = ""
            # look for a pattern match using the first line (heading)
            pos_end = content_block.find("\n")
            if pos_end != -1:
                heading_substring = content_block[0:pos_end]
            print(f'heading_substring: {heading_substring}')
            print("")
            print("")
            if heading_substring:
                print('we have substrings so searching new document.')
                # make sure we have a settings file to add it to (assumes docroot/settings.py)
                if os.path.exists(old_path):
                    current_content = old_path.read_text()
                    # with open(old_path, 'r') as output_file:
                    #     current_content = output_file.read()
                    start_pos = current_content.find(heading_substring)
                    end_pos = -1
                    new_content = None
                    if start_pos > -1:
                        end_pos = current_content.find(heading_substring, start_pos + len(heading_substring))
                        if end_pos > -1:
                            print('we found start and end positions so we are going to replace')
                            new_content = current_content[0:start_pos]
                            new_content += content_block
                            new_content += current_content[end_pos + len(heading_substring):]
                    # if we did not find a start or end pos so lets append
                    if start_pos <= -1 or end_pos <= -1:
                        print('we did not find start and or end so lets append')
                        new_content = current_content + "\n"
                        new_content += content_block + "\n"

                    # print("NEW CONTENT")
                    # print(new_content)

                    if new_content:
                        # backup the old file before we mess with it
                        datetime_ext = datetime.now().strftime("%Y%m%d-%H%M")
                        new_file_name = old_path.name + "." + datetime_ext
                        shutil.copy(old_path, old_path.parent / new_file_name)
                        # write the new_content to the current file
                        if self.testing:
                            test_file_name = old_path.stem + "_new.py"
                            test_file = old_path.parent / test_file_name
                            test_file.write_text(new_content)
                            self.stdout.write(self.style.SUCCESS(f'Successfully updated {new_path} to {test_file}'))
                        else:
                            old_path.write_text(new_content)
                            self.stdout.write(self.style.SUCCESS(f'Successfully updated {new_path} to {old_path}'))
                else:
                    self.stderr.write(self.style.ERROR(f'Settings file [{old_path}] was not found!'))
        else:
            self.stderr.write(self.style.ERROR(f'Settings append file [{new_path}] was not found!'))

    @staticmethod
    def get_module_path():
        # first try and get it from distutils (since pyenv symlinks to root version and site doesnt work for virtualenv)
        module_path = pathlib.Path(get_python_lib()) / 'docrootcms'
        print(f'module path from DISTUTILS.SYSCONFIG: {module_path}')
        if not os.path.exists(module_path):
            module_path = pathlib.Path(os.__file__).parent / 'site-packages' / 'docrootcms'
            print(f'module path from runtime: {module_path}')
        if not os.path.exists(module_path):
            # for debugging lets next try to find the app locally to copy from
            module_path = pathlib.Path('docrootcms')
            print(f'module path from project: {module_path}')
        if not os.path.exists(module_path):
            raise ModuleNotFoundError(
                'Module docrootcms was not installed.  Install using pip install django-docrootcms and try again.')
        return module_path

    def install(self):
        success_instructions = """
        Successfully Installed cms.

        """
        # check that docroot app exists
        # NOTE: we will assume if there is a docroot dir then we already installed
        if not os.path.exists('docroot'):
            self.stderr.write(self.style.ERROR('docroot application directory not found!'))
            return 'Install docroot application according to instructions: ' \
                   'https://github.com/sstacha/django-docroot-cms'
        else:
            self.stdout.write(f'installing docroot files...')
            # get the directory from docrootcms/docroot/files
            module_path = self.get_module_path()
            module_docroot_files_path = module_path / 'install' / 'files'
            local_path = pathlib.Path()
            local_docroot_files_path = local_path / 'docroot' / 'files'
            print(f'local path: {local_path}')
            shutil.copytree(module_docroot_files_path, local_docroot_files_path)
            self.stdout.write(
                self.style.SUCCESS(f'Successfully copied {module_docroot_files_path} to {local_docroot_files_path}'))
            # check that required directories exist
            if not os.path.exists(local_path / 'images'):
                os.makedirs(local_path / 'images')
                self.stdout.write(self.style.SUCCESS(f"Created {local_path / 'images'}"))
            if not os.path.exists(local_path / 'cache'):
                os.makedirs(local_path / 'cache')
                self.stdout.write(self.style.SUCCESS(f"Created {local_path / 'cache'}"))
            if not os.path.exists(local_path / 'data'):
                os.makedirs(local_path / 'data')
                self.stdout.write(self.style.SUCCESS(f"Created {local_path / 'data'}"))
            if not os.path.exists(local_path / 'docroot' / 'templates'):
                os.makedirs(local_path / 'docroot' / 'templates')
                self.stdout.write(self.style.SUCCESS(f"Created {local_path / 'docroot' / 'templates'}"))
            if not os.path.exists(local_path / 'docroot' / 'dt.inc'):
                os.makedirs(local_path / 'docroot' / 'dt.inc')
                self.stdout.write(self.style.SUCCESS(f"Created {local_path / 'docroot' / 'dt.inc'}"))
            return success_instructions

    def develop(self):
        success_instructions = """
        Successfully Installed cms application for development.

        """
        # check that docroot-cms app doesn't already exist locally
        if os.path.exists('docrootcms'):
            self.stderr.write(self.style.ERROR('docrootcms application already exists locally for development!'))
            return 'Remove the existing directory to reload a newer version.'
        else:
            self.stdout.write(f'installing docrootcms from virtual environment...')
            module_path = self.get_module_path()
            local_path = pathlib.Path() / 'docrootcms'
            print(f'local path: {local_path}')
            shutil.copytree(module_path, local_path, dirs_exist_ok=True)
            self.stdout.write(
                self.style.SUCCESS(f'Successfully copied {module_path} to {local_path}'))
            return success_instructions

    def handle(self, *args, **options):
        if "update" in options['option']:
            try:
                if "test" in options['option']:
                    self.testing = True
                self.stdout.write(self.style.WARNING(f'{self.update()}'))
            except ModuleNotFoundError as nfe:
                self.stderr.write(self.style.ERROR(f'Block not found.  This should not happen. {nfe}'))
        elif "install" in options['option']:
            try:
                if "test" in options['option']:
                    self.testing = True
                self.stdout.write(self.style.WARNING(f'{self.install()}'))
            except ModuleNotFoundError as nfe:
                self.stderr.write(self.style.ERROR(f'{nfe}'))
        elif "develop" in options['option']:
            self.stdout.write(self.style.WARNING(f'{self.develop()}'))
        else:
            self.stdout.write(self.style.SUCCESS(self.help))



# ------------------------ DOCROOT CMS SETTINGS ------------------------------------
# add our different roots for static files to be served up
import os
import pathlib
import datetime

try:
    STATIC_ROOT
except NameError:
    STATIC_ROOT = pathlib.Path(BASE_DIR, "static/")
# IMAGES_ROOT = pathlib.Path(BASE_DIR, "images/")
# CACHED_ROOT = pathlib.Path(BASE_DIR, "cache/")
DOCROOT_ROOT = pathlib.Path(BASE_DIR, "docroot/files/")
# STATICFILES_DIRS = (
#     IMAGES_ROOT,
#     CACHED_ROOT,
#     DOCROOT_ROOT,
# )
MEDIA_URL = '/media/'
MEDIA_ROOT = pathlib.Path(BASE_DIR, 'media')

# add our docroot application to the installed apps and middleware initializations
# NOTE: I am always seeing INSTALLED_APPS & MIDDLEWARE as a modifiable list; if changed to tuple this will fail!
if 'docroot' not in INSTALLED_APPS and 'docrootcms' in INSTALLED_APPS:
    # insert docroot at the beginning so we can override admin/login templates in docroot; revert to below if not
    INSTALLED_APPS.insert(0, 'docroot')
    # docrootcms_idx = INSTALLED_APPS.index('docrootcms')
    # INSTALLED_APPS.insert(docrootcms_idx, 'docroot')

if 'docroot' in INSTALLED_APPS:
    MIDDLEWARE += ('docrootcms.middleware.DocrootFallbackMiddleware',)
    if 'docroot/files/dt.inc' not in TEMPLATES[0]['DIRS'] or 'docroot/files/dt.inc/' not in TEMPLATES[0]['DIRS']:
        TEMPLATES[0]['DIRS'].append('docroot/files/dt.inc')
    if 'docrootcms.contrib.blog' not in INSTALLED_APPS:
        INSTALLED_APPS.append('docrootcms.contrib.blog')
    if 'markdownx' not in INSTALLED_APPS:
        INSTALLED_APPS.append('markdownx')
    if 'tagulous' not in INSTALLED_APPS:
        INSTALLED_APPS.append('tagulous')

    MARKDOWNX_MARKDOWN_EXTENSIONS = ['markdown.extensions.extra']
    MARKDOWNX_MEDIA_PATH = datetime.datetime.now().strftime('markdownx/%Y')
    # sas 2020-09-27 : new setting to use the original image name instead of the default uuid ones
    MARKDOWNX_USE_ORIGINAL_IMAGE_NAME = True
    # tagulous serialization modules
    SERIALIZATION_MODULES = {
        'xml':    'tagulous.serializers.xml_serializer',
        'json':   'tagulous.serializers.json',
        'python': 'tagulous.serializers.python',
        'yaml':   'tagulous.serializers.pyyaml',
    }
# else:
#     print("WARNING: 'docroot' was not found in INSTALLED_APPS so we are skipping setting up the middleware!")

# added for a problem in the way apache handles WSGI; would like to push this to web server at some point
#   to eliminate 2 file checks for every request
# don't allow our cms to serve up any templates or python code as static files; include .htaccess for good measure
# to disable checks set USE_STATIC_FORBIDDEN = False (eliminates 2 file system checks per request)
USE_STATIC_FORBIDDEN = False
STATIC_FORBIDDEN_EXTENSIONS = ['.dt', '.py', ]
STATIC_FORBIDDEN_FILE_NAMES = ['.htaccess', ]
# We tell django to not append slashes as this messes with our combined static/dynamic template pages docroot
# APPEND_SLASH = False
# Adding variable to tell our stuff to not be language aware for the default language (no /en/ appended)
#   This is helpful for troubleshooting migrated DjangoCMS pages
IGNORE_LANGUAGE_PREFIX = True
# DISABLE_AUTHENTICATION = True

# add logging and our loggers
LOGGING = {
    'version': 1,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        # 'file': {
        #     'level': 'DEBUG',
        #     'class': 'logging.FileHandler',
        #     'filename': './django.log',
        #     'formatter': 'simple'
        # },
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': True,
        },
        'docroot': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': True,
        },
    }
}
if DEBUG:
    # make all loggers use the console.
    for logger in LOGGING['loggers']:
        LOGGING['loggers'][logger]['handlers'] = ['console']
else:
    # make sure all production servers never show debugging info.  set them to info if they are debug
    for logger in LOGGING['loggers']:
        if LOGGING['loggers'][logger]['level'] == 'DEBUG':
            LOGGING['loggers'][logger]['level'] = 'INFO'

# OVERRIDING DATABASE LOCATION AND NAME FOR EASY SHARING
# NOTE: we are only overriding for sqllite3.  Move the db block below this if you don't want any overrides.
if 'default' in DATABASES and 'ENGINE' in DATABASES['default'] and DATABASES['default']['ENGINE'] == 'django.db.backends.sqlite3':
    DATABASES['default']['NAME']=pathlib.Path(BASE_DIR, 'data', 'db.sqlite3')

# OVERRIDE THE DEFAULT CACHE TO DISABLE TEMPLATE CACHING IN DEV
if DEBUG:
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
        },
    }

# SECURITY WARNING: keep the secret key used in production a secret!
# NOTE: Recommend using a DOCROOTCMS_SECRET_KEY environment variable which will get replaced at runtime below
# Replace any DOCROOTCMS_ prefixed environment variables in settings at startup
# NOTE: used for docker/local machine environment variable loading overrides
import sys
this_module = sys.modules[__name__]
env_prefix = "DOCROOTCMS_"
for k, v in os.environ.items():
    if k.upper().startswith(env_prefix):
        attr_key = k[len(env_prefix):]
        if attr_key:
            # print (f"attempting to set {attr_key} to [{str(v)}]")
            setattr(this_module, attr_key, v)

# ------------------------ DOCROOT CMS SETTINGS ------------------------------------



# from myapp.models import Web_Region

context = {'title': 'my static title',
               'description': 'my static description',
               'data': 'my static data',
               }

def get_context(request):
    # region_list = Web_Region.objects.values_list('region_name', flat=True)
    context.update({'data': 'my dynamic data'})
    return context


from django.core import serializers
from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.safestring import mark_safe
from django.contrib.auth.models import User
from tagulous.models import TagField, TagModel, TagTreeModel
from docrootcms.models import ContentMarkdownField


# BLOG MODELS
# Different categories, tags, classifications (tagulous)
class BlogTag(TagModel):
    """
    Blog Tags are public keywords that content people can use as metadata for the content.  They are visible
    to customers usually in the form of a tag cloud or keywords at the bottom to find related content.
    Ex: content has been tagged with 'technology' and shown at the bottom of the post to show other posts tagged
        with technology.
    """
    class TagMeta:
        force_lowercase = True


class BlogCategory(TagTreeModel):
    """
    Blog Categories are like tags but they are more structured and can have a hiearchy in the form of paths.  This
    has also been described as a content taxonomy.  This is often used as fixed headings and or to organize blogs for
    different website areas to display.
    Ex: content has been tagged with 'sports/NHL', 'sports/MLB' and 'tech/software/linux'
    This can be used on the main blog page to show 'sports', 'tech' etc
    Then each can have a blog with top posts for each sport type.
    NOTE: unlike tags which can be random, categories / taxonomy should be well thought out and defined beforehand
    """
    class TagMeta:
        force_lowercase = True

    class Meta:
        verbose_name_plural = "Blog categories"


class BlogProperty(TagModel):
    """
    Blog Properties are like tags for internal use.
    Ex: content has been tagged with 'homepage' and or 'hidden'
    Plugin properties = ['homepage',]
    Plugin exclude properties = ['hidden']
    Plugin will show all posts tagged 'homepage' but not any tagged 'hidden'
    Furthermore, homepage or hidden will not show up as a category or tagged as related content customers can use
    """
    class TagMeta:
        force_lowercase = True

    class Meta:
        verbose_name_plural = "Blog properties"


class MarkdownPostManager(models.Manager):
    def live(self):
        queryset = self.get_queryset()
        now = timezone.now()
        return queryset.filter(publish_date__lte=now)
    #
    # def published_recently(self):
    #     queryset = self.get_queryset()
    #     return queryset.order_by('-publish_date')


class MarkdownPost(models.Model):
    objects = MarkdownPostManager()
    title = models.CharField(max_length=250, unique=True)
    slug = models.SlugField(max_length=250,
                            unique=True,
                            help_text='SEO Friendly name that is unique for use in URL')
    teaser = ContentMarkdownField(null=True, blank=True, field_image_prefix='post/teaser', help_text=mark_safe(
        'Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>'))
    content = ContentMarkdownField(field_image_prefix='post/content', help_text=mark_safe(
        'Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>'))
    author = models.ForeignKey(User, on_delete=models.CASCADE, null=True, blank=True)
    # image = FilerImageField(null=True, blank=True)
    publish_date = models.DateTimeField(default=timezone.now)
    expire_date = models.DateTimeField(null=True, blank=True)
    categories = TagField(
        to=BlogCategory,
        blank=True,
        help_text="Splits on commas and spaces;  hierarchy is defined by paths.  "
                  "Collectively known as a 'content taxonomy'.  This is often used as headings or to organize"
                  "different blog content."
                  "Ex: 'sports/nfl' 'sports/mlb' 'tech/sofware/linux'"
                  "NOTE: unlike tags, categories should be well thought out beforehand and not change often"
    )
    tags = TagField(
        to=BlogTag,
        blank=True,
        help_text="Splits on commas and spaces; tags will be displayed to the public and used to "
                  "pull a set of posts to show.  For example, you could tag posts with 'docker' "
                  "'zfs' then the plugin will show these at the bottom for visitors to use if there"
                  "are more posts with that tag."
    )
    properties = TagField(
        to=BlogProperty,
        blank=True,
        help_text="Splits on commas and spaces; like tags but not displayed to the public."
                  "For example, you could tag posts with 'homepage' and or "
                  "'hidden' then pull all posts tagged 'homepage' but not 'hidden'"
    )
    created_date = models.DateTimeField(auto_now_add=True, editable=False)
    modified_date = models.DateTimeField(auto_now=True, editable=False)
    modified_by = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        ordering = ['-publish_date', 'title']

    def __str__(self):
        return str(self.title)

    def get_absolute_url(self):
        return reverse("blog_detail", kwargs={"slug": self.slug})

    def as_json(self):
        return serializers.serialize('json', [self])[0]

# SIMPLE POST IS JUST TEXT WITH NO CATEGORIES OR CLASSIFICATIONS
# class SimplePost(models.Model):
#     title = models.CharField(max_length=250, unique=True)
#     slug = models.SlugField(max_length=250,
#                             unique=True,
#                             help_text='SEO Friendly name that is unique for use in URL')
#     content = models.TextField()
#     author = models.ForeignKey(User, on_delete=models.CASCADE, null=True, blank=True)
#     publish_date = models.DateTimeField(default=timezone.now)
#     expire_date = models.DateTimeField(null=True, blank=True)
#     created_date = models.DateTimeField(auto_now_add=True, editable=False)
#     modified_date = models.DateTimeField(auto_now=True, editable=False)
#     modified_by = models.CharField(max_length=255, null=True, blank=True)
#
#     class Meta:
#         ordering = ['-publish_date', 'title']
#
#     def __str__(self):
#         return str(self.title)
#
#     def get_absolute_url(self):
#         return reverse("blog_detail", kwargs={"slug": self.slug})
#
# HTML POST MAY BE SOMETHING I LOOK AT IN THE FUTURE TO HAVE CKEDITOR INSTEAD OF MARKDOWN
#   but...  maybe not...
# class HTMLPost(models.Model):
#     title = models.CharField(max_length=250, unique=True)
#     slug = models.SlugField(max_length=250,
#                             unique=True,
#                             help_text='SEO Friendly name that is unique for use in URL')
#     # teaser = HTMLField(
#     #     max_length=300,
#     #     configuration='CKEDITOR_SETTINGS_SIMPLE',
#     #     help_text=u'Full text of the TD post.',
#     #     )
#     # content = HTMLField(
#     #     max_length=20000,
#     #     configuration='CKEDITOR_SETTINGS_SIMPLE',
#     #     help_text=u'Full text of the blog post.'
#     # )
#     teaser = MarkdownxField()
#     content = MarkdownxField()
#     author = models.ForeignKey(User, on_delete= models.CASCADE, null=True, blank=True)
#     # image = FilerImageField(null=True, blank=True)
#     publish_date = models.DateTimeField(default=timezone.now)
#     expire_date = models.DateTimeField(null=True, blank=True)
#     # categories = TagField(
#     #     to=BlogCategory,
#     #     blank=True,
#     #     help_text="Splits on commas and spaces;  hierarchy is defined by paths.  "
#     #               "Collectively known as a 'content taxonomy'.  This is often used as headings or to organize"
#     #               "different blog content."
#     #               "Ex: 'sports/nfl' 'sports/mlb' 'tech/sofware/linux'"
#     #               "NOTE: unlike tags, categories should be well thought out beforehand and not change often"
#     # )
#     # tags = TagField(
#     #     to=BlogTag,
#     #     blank=True,
#     #     help_text="Splits on commas and spaces; tags will be displayed to the public and used to "
#     #               "pull a set of posts to show.  For example, you could tag posts with 'docker' "
#     #               "'zfs' then the plugin will show these at the bottom for visitors to use if there"
#     #               "are more posts with that tag."
#     # )
#     # properties = TagField(
#     #     to=BlogProperty,
#     #     blank=True,
#     #     help_text="Splits on commas and spaces; like tags but not displayed to the public."
#     #               "For example, you could tag posts with 'homepage' and or "
#     #               "'hidden' then pull all posts tagged 'homepage' but not 'hidden'"
#     # )
#     created_date = models.DateTimeField(auto_now_add=True, editable=False)
#     modified_date = models.DateTimeField(auto_now=True, editable=False)
#     modified_by = models.CharField(max_length=255, null=True, blank=True)
#
#     class Meta:
#         ordering = ['-publish_date', 'title']
#
#     def __str__(self):
#         return str(self.title)
#
#     def get_absolute_url(self):
#         return reverse("blog_detail", kwargs={"slug": self.slug})
#
#     def as_json(self):
#         return serializers.serialize('json', [self])[0]
#
#     def as_html(self):
#         return markdownify(self.content)



"""
Markdown tags to convert markdown to HTML for rendering on the template
NOTE: templates will need to use |safe filter to show html and not the escaped text
"""
from django import template
from django.template.defaultfilters import stringfilter
from django.utils import timezone
import datetime
from markdownx.utils import markdownify

register = template.Library()


@register.filter
@stringfilter
def markdown_to_html(markdown_content):
    return markdownify(markdown_content)


@register.simple_tag
def time_since(timestamp=None):
    """
    Returns a humanized rstring representing time difference
    between now() and the input timestamp.

    The output rounds up to days, hours, minutes, or seconds.
    4 days 5 hours returns '4 days'
    0 days 4 hours 3 minutes returns '4 hours', etc...
    """
    rstr = ""
    if not timestamp or not isinstance(timestamp, datetime.datetime):
        return rstr

    now = timezone.now()
    timediff = now - timestamp
    days = timediff.days
    weeks = days//7
    months = days//30
    minutes = timediff.seconds % 3600 // 60
    seconds = timediff.seconds % 3600 % 60
    hours = minutes // 60

    if days > 365:
        return "> a year"
    if months > 0:
        if months == 1:
            tstr = "month"
        else:
            tstr = "months"
        rstr = rstr + "%s %s" % (months, tstr)
        return rstr
    if weeks > 0:
        if weeks == 1:
            tstr = "week"
        else:
            tstr = "weeks"
        rstr = rstr + "%s %s" % (weeks, tstr)
        return rstr
    if days > 0:
        if days == 1:
            tstr = "day"
        else:
            tstr = "days"
        rstr = rstr + "%s %s" % (days, tstr)
        return rstr
    elif hours > 0:
        if hours == 1:
            tstr = "hour"
        else:
            tstr = "hours"
        rstr = rstr + "%s %s" % (hours, tstr)
        return rstr
    elif minutes > 0:
        if minutes == 1:
            tstr = "min"
        else:
            tstr = "mins"
        rstr = rstr + "%s %s" % (minutes, tstr)
        return rstr
    elif seconds > 0:
        if seconds == 1:
            tstr = "sec"
        else:
            tstr = "secs"
        rstr = rstr + "%s %s" % (seconds, tstr)
        return rstr
    else:
        return "Now"



# Generated by Django 3.1 on 2020-10-02 20:35

from django.db import migrations
import docrootcms.models


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='markdownpost',
            name='content',
            field=docrootcms.models.ContentMarkdownField(help_text='Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>'),
        ),
        migrations.AlterField(
            model_name='markdownpost',
            name='teaser',
            field=docrootcms.models.ContentMarkdownField(blank=True, help_text='Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>', null=True),
        ),
    ]



import re
from django.conf import settings
from django.http import JsonResponse
from datetime import datetime

# static context definition
context = {}


# dynamic context return
# NOTE: don't forget to use .update to add/replace instead of = which will ignore static definition
def get_context(request):
    regex_http_ = re.compile(r'^HTTP_.+$')
    regex_content_type = re.compile(r'^CONTENT_TYPE$')
    regex_content_length = re.compile(r'^CONTENT_LENGTH$')

    request_headers = {}
    for header in request.META:
        if regex_http_.match(header) or regex_content_type.match(header) or regex_content_length.match(header):
            request_headers[header] = request.META[header]

    b = getattr(settings, "BOOGER", "")
    b2 = getattr(settings, "BOOGER2", "")
    request_ip = request.META.get('REMOTE_ADDR')
    real_ip = request.META.get('HTTP_X_REAL_IP')
    context.update({'headers': request_headers, 'request_ip': str(request_ip), 'real_ip': str(real_ip), 'b': b, 'b2': b2})
    return context


# web service definitions
# NOTE: if not defined or commented will return 405-method not supported if called
#   if no data file or no web service methods defined returns 404-not found
# BEST PRACTICE: GET does not update only reads and returns data
def GET(request):
    ctx = get_context(request)
    now = datetime.now()
    ctx['freshness_date'] = str(now)
    return JsonResponse(ctx, safe=False)
#
#
# BEST PRACTICE: POST inserts/updates a single record and returns data/response (form post encoded body)
# def POST(request):
#     pass
#
#
# BEST PRACTICE: PUT inserts/updates record(s) and returns data/response (various body formats like json/xml etc)
# def PUT(request):
#     pass
#
#
# BEST PRACTICE: DELETE deletes (maybe soft delete like marking record deleted) record and returns data/response
#   best practice is to pass just unique identifier not a lot complete data set
# def DELETE(request):
#     pass



# Generated by Django 3.1 on 2020-08-27 17:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import markdownx.models
import tagulous.models.fields
import tagulous.models.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='BlogCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField()),
                ('count', models.IntegerField(default=0, help_text='Internal counter of how many times this tag is in use')),
                ('protected', models.BooleanField(default=False, help_text='Will not be deleted when the count reaches 0')),
                ('path', models.TextField()),
                ('label', models.CharField(help_text='The name of the tag, without ancestors', max_length=255)),
                ('level', models.IntegerField(default=1, help_text='The level of the tag in the tree')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='blog.blogcategory')),
            ],
            options={
                'verbose_name_plural': 'Blog categories',
            },
            bases=(tagulous.models.models.BaseTagTreeModel, models.Model),
        ),
        migrations.CreateModel(
            name='BlogProperty',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField()),
                ('count', models.IntegerField(default=0, help_text='Internal counter of how many times this tag is in use')),
                ('protected', models.BooleanField(default=False, help_text='Will not be deleted when the count reaches 0')),
            ],
            options={
                'verbose_name_plural': 'Blog properties',
            },
            bases=(tagulous.models.models.BaseTagModel, models.Model),
        ),
        migrations.CreateModel(
            name='BlogTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField()),
                ('count', models.IntegerField(default=0, help_text='Internal counter of how many times this tag is in use')),
                ('protected', models.BooleanField(default=False, help_text='Will not be deleted when the count reaches 0')),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
                'unique_together': {('slug',)},
            },
            bases=(tagulous.models.models.BaseTagModel, models.Model),
        ),
        migrations.CreateModel(
            name='MarkdownPost',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250, unique=True)),
                ('slug', models.SlugField(help_text='SEO Friendly name that is unique for use in URL', max_length=250, unique=True)),
                ('teaser', markdownx.models.MarkdownxField(blank=True, help_text='Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>', null=True)),
                ('content', markdownx.models.MarkdownxField(help_text='Markdown Reference: <a href="https://commonmark.org/help/">https://commonmark.org/help/</a>')),
                ('publish_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('expire_date', models.DateTimeField(blank=True, null=True)),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('modified_date', models.DateTimeField(auto_now=True)),
                ('modified_by', models.CharField(blank=True, max_length=255, null=True)),
                ('author', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('categories', tagulous.models.fields.TagField(_set_tag_meta=True, blank=True, force_lowercase=True, help_text="Splits on commas and spaces;  hierarchy is defined by paths.  Collectively known as a 'content taxonomy'.  This is often used as headings or to organizedifferent blog content.Ex: 'sports/nfl' 'sports/mlb' 'tech/sofware/linux'NOTE: unlike tags, categories should be well thought out beforehand and not change often", to='blog.BlogCategory', tree=True)),
                ('properties', tagulous.models.fields.TagField(_set_tag_meta=True, blank=True, force_lowercase=True, help_text="Splits on commas and spaces; like tags but not displayed to the public.For example, you could tag posts with 'homepage' and or 'hidden' then pull all posts tagged 'homepage' but not 'hidden'", to='blog.BlogProperty')),
                ('tags', tagulous.models.fields.TagField(_set_tag_meta=True, blank=True, force_lowercase=True, help_text="Splits on commas and spaces; tags will be displayed to the public and used to pull a set of posts to show.  For example, you could tag posts with 'docker' 'zfs' then the plugin will show these at the bottom for visitors to use if thereare more posts with that tag.", to='blog.BlogTag')),
            ],
            options={
                'ordering': ['-publish_date', 'title'],
            },
        ),
    ]



import os
import logging
import mimetypes
import importlib.util
import json
import codecs
import base64
from django.conf import settings
# from django.template import Context
from django.template import Template, Origin, RequestContext
from django.http import HttpResponse, FileResponse, JsonResponse, HttpResponseForbidden, HttpResponseRedirect
from django.views.generic import View
from django.views.decorators.csrf import csrf_protect
from django.core import serializers
# from pprint import pprint
# from django.shortcuts import redirect
from django.shortcuts import render
from django.contrib import messages
from django.contrib.auth import authenticate, login

from .forms import MarkdownImageUploadForm
from markdownx.views import ImageUploadView

from .forms import LoginForm
from .models import Content

log = logging.getLogger("cms.views")


# Markdownx overridden views to have images store the way we want
class MarkdownImageUploadView(ImageUploadView):
    form_class = MarkdownImageUploadForm


# This view is called from DocrootFallbackMiddleware.process_response
# when a 404 is raised and we are not working with a template (we want to look for a static file in the docroot)
def static(request):
    docroot_dir = getattr(settings, "DOCROOT_ROOT", "")
    use_static = getattr(settings, "USE_STATIC_FORBIDDEN", False)
    log.debug("docroot dir: " + str(docroot_dir))
    path = request.path_info
    if path.startswith("/"):
        path = path[1:]
    log.debug("path: " + path)
    file = os.path.join(docroot_dir, path)
    log.debug("file: " + file)
    if os.path.isfile(file):
        # for various reasons we don't want to serve up various file extensions. Let's look at a setting containing
        # extensions to ignore
        # USE CASE: Apache at root passes .htaccess, .dt and .py files through we don't want to show these static files
        if use_static:
            forbidden_extensions = getattr(settings, "STATIC_FORBIDDEN_EXTENSIONS", [])
            forbidden_file_names = getattr(settings, "STATIC_FORBIDDEN_FILE_NAMES", [])
            filename, ext = os.path.splitext(file)
            if ext in forbidden_extensions:
                return HttpResponseForbidden()
            elif os.path.basename(filename) in forbidden_file_names:
                return HttpResponseForbidden()
        log.debug("found static file: " + file)
        log.debug("downloading...")
        response = FileResponse(open(file, 'rb'), content_type=mimetypes.guess_type(path)[0])
        return response
    else:
        return None


# This view is called from DocrootFallbackMiddleware.process_response
# when a 404 is raised, which often means CsrfViewMiddleware.process_view
# has not been called even if CsrfViewMiddleware is installed. So we need
# to use @csrf_protect, in case the template needs {% csrf_token %}.
# However, we can't just wrap this view; if no matching page exists,
# or a redirect is required for authentication, the 404 needs to be returned
# without any CSRF checks. Therefore, we only
# CSRF protect the internal implementation (render page).
def page(request):
    # NEW VERSION: pushed meta processing down to class TemplateMeta
    meta = TemplateMeta(request)
    # NOTE: only calls render_page if the template is found
    return meta.render()


# This view is called from DocrootFallbackMiddleware.process_response
# when a 404 is raised. We do not need to use @csrf_protect since a web service should never contain input forms.
def api(request):
    meta = ApiMeta(request)
    return meta.render()


class TemplateMeta:
    """
        encapsulates the core atts and methods to get a valid template
    """

    def __init__(self, request):
        # setup our basic attributes for the meta-data we will use for validation and template creation
        self.is_found = False
        self.request = request
        self.docroot_dir = getattr(settings, "DOCROOT_ROOT", "")
        log.debug("docroot dir: " + str(self.docroot_dir))
        self.original_path = request.path_info.strip()
        self.path = self.original_path
        if self.path.startswith("/"):
            self.path = self.path[1:]
        self.file_name = os.path.join(self.docroot_dir, self.path)
        log.debug("file: " + str(self.file_name))
        self.module_name = self.path
        self.template_name = self.path
        # try and modify urls for logic on how to pull the correct template
        self.find_template()

        # get our settings needed for processing defaulting if they are not there
        ignore_lanaguage_prefix = getattr(settings, 'IGNORE_LANGUAGE_PREFIX', False)
        append_slash = getattr(settings, 'APPEND_SLASH', False)

        if not ignore_lanaguage_prefix:
            # if not found lets try and make sure it is not because of language
            if not self.is_found and request.LANGUAGE_CODE:
                # new code to make us django language aware (strip language code when looking for a template)
                # print('request lang: ' + str(request.LANGUAGE_CODE))
                # print('settings lang: ' + str(settings.LANGUAGE_CODE))
                # print('languages: ' + str(settings.LANGUAGES))
                lang = '/' + request.LANGUAGE_CODE + "/"
                if self.original_path.startswith(lang):
                    self.path = self.original_path[len(lang):]
                    self.file_name = os.path.join(self.docroot_dir, self.path)
                    log.debug("language stripped file: " + str(self.file_name))
                    self.module_name = self.path
                    self.template_name = self.path
                    # re-try and modify urls for logic on how to pull the correct template
                    self.find_template()

            # finally if we still don't have a template and ends with / and APPEND_SLASH is set and False strip it
            if not self.is_found and request.LANGUAGE_CODE and append_slash and self.original_path.endswith('/'):
                lang = '/' + request.LANGUAGE_CODE + "/"
                if self.original_path.startswith(lang):
                    # same as above except now try to strip the last slash
                    self.path = self.original_path[len(lang):len(self.original_path) - 1]
                    self.file_name = os.path.join(self.docroot_dir, self.path)
                    log.debug("language stripped file: " + str(self.file_name))
                    self.module_name = self.path
                    self.template_name = self.path

                    # re-try and modify urls for logic on how to pull the correct template
                    self.find_template()

    # contains the logic for taking a request url and attempting to locate a page template for it
    def find_template(self):
        # if the url ends in .html then try to load a corresponding template from the docroot/files directory
        if self.file_name.endswith(".html"):
            # our url will request .html but we want to look for a .dt file (required for template processing)
            self.file_name = self.file_name[:-4]
            self.file_name += "dt"
            self.template_name = self.template_name[:-4]
            self.template_name += "dt"
            if os.path.isfile(self.file_name):
                log.debug("found file: " + str(self.file_name))
                self.is_found = True

        elif self.file_name.endswith('/'):
            self.file_name += "index.dt"
            if os.path.isfile(self.file_name):
                log.debug("found file: " + str(self.file_name))
                self.module_name += "index.html"
                self.template_name += "index.dt"
                self.is_found = True

        else:
            self.file_name += ".dt"
            if os.path.isfile(self.file_name):
                log.debug("found file: " + str(self.file_name))
                self.module_name += ".html"
                self.template_name += ".dt"
                self.is_found = True

    def render(self):
        if self.is_found:
            log.debug("opening file: " + str(self.file_name))
            fp = codecs.open(self.file_name, "r", encoding='utf-8')
            log.debug("loading template...")
            # template = Template(fp.read().encode('utf-8'), Origin(self.file_name), self.template_name)
            # sas django 2.2 no longer reqiures bytes so we can go back to just reading it in
            # if this has problems with utf-8 content then do a decode afterwards instead
            template = Template(fp.read(), Origin(self.file_name), self.template_name)
            log.debug("closing file")
            fp.close()

            if template:
                log.debug("attempting to load context and render the template...")
                return render_page(self.request, template, self.module_name)
            else:
                return None

    def is_found(self):
        return self.is_found

    def __str__(self):
        return self.file_name


class ApiMeta:
    """
        encapsulates the core atts and methods to get a valid api data file and process it
    """

    def __init__(self, request):
        self.ALL_OPTIONS = ['GET', 'POST', 'PUT', 'TRACE', 'DELETE', 'HEAD', 'PATCH']
        self.options = []
        # setup our basic attributes for the meta-data we will use for validation and api creation
        self.is_found = False
        self.request = request
        self.docroot_dir = getattr(settings, "DOCROOT_ROOT", "")
        log.debug("docroot dir: " + str(self.docroot_dir))
        self.original_path = request.path_info.strip()
        self.path = self.original_path
        if self.path.startswith("/"):
            self.path = self.path[1:]
        self.file_name = os.path.join(self.docroot_dir, self.path)
        log.debug("file: " + str(self.file_name))
        self.api_name = self.path
        # try and modify urls for logic on how to pull the correct template
        self.find_api()

        # get our settings needed for processing defaulting if they are not there
        ignore_lanaguage_prefix = getattr(settings, 'IGNORE_LANGUAGE_PREFIX', False)
        append_slash = getattr(settings, 'APPEND_SLASH', False)

        if not ignore_lanaguage_prefix:
            # if not found lets try and make sure it is not because of language
            if not self.is_found and request.LANGUAGE_CODE:
                # new code to make us django language aware (strip language code when looking for a template)
                # print('request lang: ' + str(request.LANGUAGE_CODE))
                # print('settings lang: ' + str(settings.LANGUAGE_CODE))
                # print('languages: ' + str(settings.LANGUAGES))
                lang = '/' + request.LANGUAGE_CODE + "/"
                if self.original_path.startswith(lang):
                    self.path = self.original_path[len(lang):]
                    self.file_name = os.path.join(self.docroot_dir, self.path)
                    log.debug("language stripped file: " + str(self.file_name))
                    self.api_name = self.path
                    # re-try and modify urls for logic on how to pull the correct template
                    self.find_api()

            # finally if we still don't have an api and ends with / and APPEND_SLASH is set try without language prefix
            # NOTE: this is needed because if APPEND_SLASH is set when django is finished processing we can have a
            #   unexpected slash EX: request-> /test/index.json | after django -> /test/index.json/
            if not self.is_found and request.LANGUAGE_CODE and append_slash and self.original_path.endswith('/'):
                lang = '/' + request.LANGUAGE_CODE + "/"
                if self.original_path.startswith(lang):
                    # same as above except now try to strip the last slash
                    self.path = self.original_path[len(lang):len(self.original_path) - 1]
                    self.file_name = os.path.join(self.docroot_dir, self.path)
                    log.debug("language stripped file: " + str(self.file_name))
                    self.api_name = self.path
                    # re-try without the language code
                    self.find_api()

    # contains the logic for taking a request url and attempting to locate an api for it
    def find_api(self):
        # if the url ends in .json then try to load a corresponding api from the docroot/files directory
        if self.file_name.endswith(".json"):
            # our url will request .json but we want to look for a .data.py file
            self.file_name = self.file_name[:-4]
            self.file_name += "data.py"
            if os.path.isfile(self.file_name):
                log.debug("found file: " + self.file_name)
                self.is_found = True

    def render(self):
        # return none if not found
        if self.is_found:
            # try to load a data file if it is there in order to get the context
            # all data files should support get_context() or a context property
            try:
                log.debug("attempting to load data_file...")
                spec = importlib.util.spec_from_file_location(self.api_name, self.file_name)
                data = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(data)
                # datafile = imp.load_source(module_name, datafile_name)
                # note changing datafile below to data
            except FileNotFoundError:
                return None
            except Exception as ex:
                data = None
                logging.error(str(ex))
                if settings.DEBUG:
                    raise ex

            if data:
                methods = dir(data)
                for method in methods:
                    if method in self.ALL_OPTIONS:
                        self.options.append(method)
                # figure out the proper method to call (get, post trace etc) return method not supported if not there
                request_method = self.request.method
                # adding method overriding; very useful for testing or getting around proxies
                # look for a GET or POST "_method" parameter and change the method we are looking for if found
                override = self.request.GET.get('_method')
                if not override:
                    override = self.request.POST.get('_method')
                if override:
                    request_method = override.upper().strip()
                    log.debug(f"Overriding method [{self.request.method}] with parameter value [{request_method}]")
                try:
                    initmethod = getattr(data, request_method)
                except AttributeError:
                    initmethod = None
                if initmethod:
                    # we may want to return something like a redirect so if is response then return it; else use for
                    # data!
                    content = initmethod(self.request)
                    if isinstance(content, HttpResponse):
                        return content
                    else:
                        response = HttpResponse(content)
                        response['Allow'] = ",".join(self.options)
                        # response['Content-Type'] = "application/json"
                        return response
                else:
                    log.error(
                        "Found datafile [" + self.file_name + "] but didn't find method [" + request_method + "]!")
                    # if we don't support any methods lets return a not found instead of method not supported
                    #   since there is no way to adjust the call to get it to work
                    if not self.options:
                        return None
                    response = HttpResponse("Method Not Supported [" + request_method + "]!", status=405)
                    response['Allow'] = ",".join(self.options)
                    # response['Content-Type'] = "text/plain"
                    return response

    def is_found(self):
        return self.is_found

    def __str__(self):
        return self.file_name


@csrf_protect
def render_page(request, template, module_name):
    """
    Internal interface to the dev page view.
    """
    context = {}
    log.debug("template name: " + template.name)
    log.debug("module_name: " + module_name)
    datafile_name = template.origin.name
    # strip off the html and try data.py
    if datafile_name.endswith('dt'):
        datafile_name = datafile_name[0:len(datafile_name) - 2]
        datafile_name += 'data.py'
        log.debug("datafilename: " + datafile_name)
    # else:
    #     datafile_name += '.data.py'
    # try to load a data file if it is there in order to get the context
    # all data files should support get_context() or a context property
    try:
        log.debug("attempting to load data_file...")
        spec = importlib.util.spec_from_file_location(module_name, datafile_name)
        data = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(data)

        # datafile = imp.load_source(module_name, datafile_name)
        # note changing datafile below to data
    except FileNotFoundError:
        data = None
    except Exception as ex:
        # logging.error(traceback.format_exc())
        data = None
        if settings.DEBUG:
            raise ex
    if data:
        try:
            initmethod = getattr(data, 'get_context')
        except AttributeError:
            initmethod = None
        if initmethod:
            # we may want to return something like a redirect so if is response then return it; else use for data!
            context = initmethod(request)
            if isinstance(context, HttpResponse):
                return context
        else:
            try:
                context = getattr(data, 'context')
            except AttributeError:
                context = {}
    # print("context string: " + str(context))
    template_context = RequestContext(request)
    if context:
        template_context.push(context)
    response = HttpResponse(template.render(template_context))
    # add a canonical header if we are doing some fancy string replacement so analytics work properly with SEO
    if request.path != "/" + module_name and request.scheme and request.get_host():
        response['Link'] = f'< {request.scheme}://{request.get_host()}/{module_name} >; rel="canonical"'

    return response


# class based view for getting and putting cms content
class ContentApi(View):
    def get(self, request):
        db_content = None
        # get the requested uri
        uri = request.GET.get('uri', None)
        if uri:
            db_content = Content.objects.filter(uri=uri).values('uri', 'element_id', 'content')
        # return any content for this url
        return JsonResponse(list(db_content) or [], safe=False)

    def post(self, request):
        # save and return any content for this url
        received_json_data = json.loads(request.body.decode("utf-8"))
        # data = request.body.decode('utf-8')
        # received_json_data = json.loads(data)
        content = received_json_data.get('content', None)
        if content:
            uri = received_json_data.get('uri', '')
            element_id = received_json_data.get('element_id', None)
            # lookup a record for this uri, element_id combination
            try:
                db_content = Content.objects.get(uri=uri, element_id=element_id)
            except Content.DoesNotExist:
                db_content = None

            if db_content:
                db_content.content = content
                db_content.save()
            else:
                db_content = Content()
                db_content.uri = uri
                db_content.element_id = element_id
                db_content.content = content
                db_content.save()

            serialized_object = serializers.serialize('json', [db_content, ])
            return JsonResponse(serialized_object or [], safe=False)
        else:
            return HttpResponse(status=204)


# # AUTHENTICATION VIEWS
#
class LoginFormView(View):
    form_class = LoginForm

    # initial = {'key': 'value'}
    template_name = 'login.dt'

    def get(self, request, *args, **kwargs):
        # try to get target location from header
        target = self.request.META.get('HTTP_X_TARGET')
        auth_message = self.request.META.get('HTTP_X_AUTH_MESSAGE')
        print(f"target from header: {target}")
        if not target:
            target = request.META.get('HTTP_REFERER')
            print(f"target from referrer: {target}")
        # if not target:
        #     return HttpResponseForbidden()
        # form = self.form_class(initial=self.initial)
        form = self.form_class(initial={'target': target})
        return render(request, self.template_name, {'form': form, 'auth_message': auth_message})

    def post(self, request, *args, **kwargs):
        form = self.form_class(request.POST)
        if form.is_valid():
            login = form.cleaned_data['login']
            password = form.cleaned_data['password']
            target = form.cleaned_data['target']
            print(f"target from form: {target}")
            if not target:
                target = '/'
            print(f"final target: {str(form.cleaned_data['target'])}")
            # if we have all our values set the header

            if login and password:

                auth_str = str(login) + ":" + str(password)
                auth_cookie = base64.urlsafe_b64encode(auth_str.encode("utf-8"))
                response = HttpResponseRedirect(target)
                response.set_cookie('nginxauth', auth_cookie.decode("utf-8"), httponly=True)
                return response
            else:
                messages.add_message(request, messages.ERROR, 'Login and password must not be blank!')
        return render(request, self.template_name, {'form': form})


class LogoutView(View):
    form_class = LoginForm

    # initial = {'key': 'value'}
    template_name = 'login.dt'

    def get(self, request, *args, **kwargs):
        # try to get target location from header
        target = self.request.META.get('HTTP_X_TARGET')
        print("form get")
        print(f"target from header: {target}")
        if not target:
            target = request.META.get('HTTP_REFERER')
            print(f"target from referrer: {target}")
        # if not target:
        #     return HttpResponseForbidden()
        # form = self.form_class(initial=self.initial)
        form = self.form_class(initial={'target': target})
        return render(request, self.template_name, {'form': form})

    def post(self, request, *args, **kwargs):
        form = self.form_class(request.POST)
        if form.is_valid():
            login = form.cleaned_data['login']
            password = form.cleaned_data['password']
            target = form.cleaned_data['target']
            print(f"target from form: {target}")
            if not target:
                target = '/'
            print(f"final target: {str(form.cleaned_data['target'])}")
            # if we have all our values set the header

            if login and password:

                auth_cookie = base64.urlsafe_b64encode((str(login) + ":" + str(password)).encode("ascii"))
                response = HttpResponseRedirect(target)
                response.set_cookie('nginxauth', auth_cookie, httponly=True)
                return response
            else:
                messages.add_message(request, messages.ERROR, 'Login and password must not be blank!')
        return render(request, self.template_name, {'form': form})


class AuthenticateView(View):
    form_class = LoginForm

    # initial = {'key': 'value'}
    template_name = 'login.dt'

    def get(self, request, *args, **kwargs):
        # get our header variables
        realm = request.META.get('HTTP_X_LDAP_REALM', 'Restricted')
        url = request.META.get('HTTP_X_LDAP_URL')
        start_tls = request.META.get('HTTP_X_LDAP_STARTTLS', 'false')
        disable_referrals = request.META.get('HTTP_X_LDAP_DISABLEREFERRALS', 'false')
        base_dn = request.META.get('HTTP_X_LDAP_BASEDN')
        template = request.META.get('HTTP_X_LDAP_TEMPLATE', '(cn=%(username)s)')
        bind_dn = request.META.get('HTTP_X_LDAP_BINDDN', '')
        bind_password = request.META.get('HTTP_X_LDAP_BINDPASS', '')
        cookie_name = request.META.get('HTTP_X_LDAP_COOKIENAME', 'nginxauth')
        target = self.request.META.get('HTTP_X_TARGET')
        authn_header = self.request.META.get('HTTP_WWW_AUTHENTICATE')
        auth_header = self.request.META.get('HTTP_AUTHORIZATION')
        auth_cookie = self.request.COOKIES.get(cookie_name)

        print(f"target from header: {target}")
        if not target:
            target = request.META.get('HTTP_REFERER')
            print(f"target from referrer: {target}")

        # if our authorization header is blank check if we have a cookie and if so set the authorization header
        #   from cookie and continue to achieve auto-login
        if not auth_header and auth_cookie:
            auth_header = "Basic " + auth_cookie

        # if we don't have auhorization header still then tell nginx to have the user login
        if auth_header is None or not auth_header.lower().startswith('basic '):
            # messages.info(request, "Please log in")
            # request.session['SAS_LOGIN_MESSAGE'] = 'Please log in'
            print('no auth_header so telling nginx to authenticate...')
            response = HttpResponse('Unauthorized', status=401)
            response['Cache-Control'] = 'no-cache'
            response['WWW-Authenticate'] = 'Basic realm="' + realm + '"'
            # response['X-Target'] = target
            response['X-Auth-Message'] = "Please log in"
            return response

        # we have a auth header so lets try to authenticate using the stored credentials
        #   NOTE: my have expired ect.  we need to add that next
        #   TODO: add check for expired credentials and to try again (session timeout)
        # get the username and password and attempt to authenticate
        username = self.get_username(auth_header)
        password = self.get_password(auth_header)
        # NOTE: using django for now; change to ldap later
        #   TODO: change to ldap?
        user = authenticate(username=username, password=password)
        if user is not None and user.is_active:
            login(request, user)
        else:
            # give error message and re-authenticate to login to show error
            if user is None:
                msg = "Login or Password is incorrect."
            else:
                msg = "Account has been de-activated."
            messages.add_message(request, messages.ERROR, msg)
            response = HttpResponse('Unauthorized', status=401)
            response['Cache-Control'] = 'no-cache'
            response['WWW-Authenticate'] = 'Basic realm="' + realm + '"'
            # response['X-Target'] = target
            response['X-Auth-Message'] = msg
            return response

        msg = "Authenticated"
        # todo: add processing here to determine if the url is protected and we have rights (authorization)
        # todo: add processing to determine what headers to pack in auth headers and send for this url (post authz)
        # if we got this far we should have a logged in user; lets create a global header and redirect to target
        response = HttpResponse()
        response['X-Auth-Headers'] = 'sm_constitid:3074952'
        response['X-Auth-Message'] = msg
        return response

    def get_username(self, auth_token):
        trim_token = auth_token[6:]
        btrim_token = trim_token.encode("utf-8")
        bauth_str = base64.urlsafe_b64decode(btrim_token)
        auth_str = bauth_str.decode("utf-8")
        print('decoded trimmed token: ' + str(auth_str))
        username, password = auth_str.split(':', 1)
        return username

    def get_password(self, auth_token):
        # may split this but for now use urlsafe b64 encoding
        trim_token = auth_token[6:]
        btrim_token = trim_token.encode("utf-8")
        bauth_str = base64.urlsafe_b64decode(btrim_token)
        auth_str = bauth_str.decode("utf-8")
        print('decoded trimmed token: ' + str(auth_str))
        username, password = auth_str.split(':', 1)
        return password

# # HELPER API VIEWS  (move?)
# Idea is to use this for links enhancement in ckeditor (gives immediate feedback when creating a link)
# redo this after we figure out about the requests requirement
# class UrlValidationApi(View):
#     def get(self, request):
#         # we will attempt to validate every URL parameter passed in
#
#         is_valid = True
#
#         dict_buffer = {}
#         # list_dicts = []
#
#         items = request.GET.items()
#         if not items:
#             items = request.POST.items()
#         for _key, _val in items:
#             # build a json list and return it
#             print(_key, _val)
#             check_response = False
#             check_status = 500
#
#             try:
#                 r = requests.head(_val, allow_redirects=True)
#
#                 if r.status_code >= 200 and r.status_code < 400:
#                     check_status = r.status_code
#                     # if we were intercepted by a login screen then we treat that as a forbidden instead of success
#                     if r.request.path_url.lower().startswith('/en/admin/login/' or r.request.path_url.lower(
#                     ).startswith('/admin/login/')):
#                         # print('starts with login')
#                         check_response = False
#                         is_valid = False
#                         check_status = 403
#                     else:
#                         check_response = True
#                 else:
#                     check_response = False
#                     is_valid = False
#
#                 dict_buffer[_key] = {}
#                 dict_buffer[_key]["response"] = check_response
#                 dict_buffer[_key]["status_code"] = check_status
#                 # list_dicts.append(dict_buffer)
#
#             except Exception as e:
#                 is_valid = False
#                 dict_buffer[_key] = {}
#                 dict_buffer[_key]["response"] = False
#                 dict_buffer[_key]["status_code"] = 500
#                 # list_dicts.append(dict_buffer)
#
#         # myurl = request.GET.get("myurl", None)
#         # # try and do a lookup for the url passed
#         # myresponse = False
#         # mystatus = None
#         # try:
#         #     r = requests.head(myurl)
#         #     if r.status_code >= 200 and r.status_code < 400:
#         #         myresponse = True
#         #     mystatus=r.status_code
#         # except Exception as e:
#         #     mystatus=500
#         return JsonResponse({'valid': is_valid, 'results': dict_buffer})



from django.db import models
from markdownx.models import MarkdownxField
from markdownx.models import MarkdownxFormField


# Markdownx Model overrides to use our new properties without importing all the code
class ContentMarkdownField(MarkdownxField):
    def __init__(self, *args, **kwargs):
        """
        sas 2020-09-15 : we want to add the ability to prefix our images based on the model/field via a new
            attribute 'field_image_prefix' which will be appended to the prefix defined in the settings file if given;
            this will allow us to categorize our images better based on model/field.
            EX: /media/markdownx/blog/content/me.jpg -or- /media/markdownx/blog/teaser/me.jpg
            NOTE: if not provided, will follow the same original rules as before
        """
        # pop the custom kwarg for the optional field image prefix so super does not error
        self.field_image_prefix = kwargs.pop('field_image_prefix', None)
        super(MarkdownxField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """
        Customising the ``form_class``.

        :return: TextField with a custom ``form_class``.
        :rtype: django.db.models.TextField
        """
        defaults = {'form_class': MarkdownxFormField}
        defaults.update(kwargs)
        frmfield = super(MarkdownxField, self).formfield(**defaults)
        # sas 2020-09-15 : adding attrs if not there and then adding the image_prefix from the model or empty string
        if not frmfield.widget.attrs:
            frmfield.widget.attrs = {}
        frmfield.widget.attrs['data-field-image-prefix'] = self.field_image_prefix or ""
        return frmfield


# cms models
class Content(models.Model):
    uri = models.CharField(max_length=2000, blank=True)
    element_id = models.CharField(max_length=500, null=True, blank=True)
    content = models.TextField()
    date_created = models.DateTimeField(auto_now_add=True, editable=False)
    date_modified = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        unique_together = ['uri', 'element_id']

    def __str__(self):
        return self.uri + " " + str(self.element_id)





# ------------------------ DOCROOT CMS URLS ------------------------------------
# add our different urls for the cms to work
from django.conf import settings
from django.conf.urls import include
from django.conf.urls.static import static
from docrootcms.views import MarkdownImageUploadView

urlpatterns += [
    path('_cms/', include('docrootcms.urls')),
    path('blog/', include('docrootcms.contrib.blog.urls')),
    # sas 2020-09-27 : override the markdownx url and apply our overridden view/form instead
    # NOTE: don't forget to copy the js changes into static for transferring the new field to the view
    path('markdownx/upload/', MarkdownImageUploadView.as_view(), name='markdownx_upload'),
    path('markdownx/', include('markdownx.urls')),
]
urlpatterns += static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
# ------------------------ DOCROOT CMS URLS ------------------------------------



# Generated by Django 3.0.8 on 2020-07-04 03:44

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Content',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uri', models.CharField(blank=True, max_length=2000)),
                ('element_id', models.CharField(blank=True, max_length=500, null=True)),
                ('content', models.TextField()),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
            ],
            options={
                'unique_together': {('uri', 'element_id')},
            },
        ),
    ]



from django.conf import settings
from django.utils.timezone import activate
# from django.core.exceptions import MiddlewareNotUsed
import pytz


class TZMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        # NOTE: I decided not to auto load this since everyone may want to set up timezones differently
        #   todo: add to both projects instructions
        # if 'docrootcms' in settings.INSTALLED_APPS:
        #     raise MiddlewareNotUsed("Disabling TZMiddleware because docrootcms in INSTALLED_APPS")
        if settings.DEFAULT_TZ:
            print(f'TZMiddleware defaulting timezone to [{settings.DEFAULT_TZ}]')

    def __call__(self, request):
        # start by activating our default tz if defined; we can override on call later if they have a profile set
        if settings.DEFAULT_TZ:
            activate(pytz.timezone(settings.DEFAULT_TZ))
        return self.get_response(request)
